import React, { useState, useEffect } from 'react';
import { FaChevronLeft, FaChevronRight, FaFastBackward, FaFastForward, FaSort, FaSortUp, FaSortDown } from 'react-icons/fa'; // Icons for sorting
import { NoAvailable } from '../../../assets/icon';

const BASE_URL = process.env.REACT_APP_BASE_URL
const LOCAL_URL = 'http://127.0.0.1:8000'
const API_KEY = process.env.REACT_APP_API_KEY
// Pagination Component
const Pagination = ({ currentPage, totalPages, onPageChange }) => {
  const handlePrev = () => {
      if (currentPage > 1) {
          onPageChange(currentPage - 1);
      }
  };

  const handleNext = () => {
      if (currentPage < totalPages) {
          onPageChange(currentPage + 1);
      }
  };

  const handleFirst = () => {
      onPageChange(1); // Go to first page
  };

  const handleLast = () => {
      onPageChange(totalPages); // Go to last page
  };

  // Logic to display only 3 pages in pagination
  const getPaginationRange = () => {
      const range = [];
      const totalPagesCount = totalPages;

      let start = currentPage - 1;
      let end = currentPage + 1;

      // Adjust start and end if near the boundaries
      if (currentPage === 1) {
          start = 1;
          end = Math.min(3, totalPagesCount);
      } else if (currentPage === totalPages) {
          start = Math.max(totalPagesCount - 2, 1);
          end = totalPagesCount;
      }

      for (let i = start; i <= end; i++) {
          range.push(i);
      }

      return range;
  };

  const pageRange = getPaginationRange();

  return (
      <div className="pagination-container d-flex justify-content-end mt-4">
          {/* First Page Button */}
          <button
              className="btn"
              onClick={handleFirst}
              disabled={currentPage === 1}
          >
              <FaFastBackward /> {/* Double Arrow Left */}
          </button>

          <button
              className="btn"
              onClick={handlePrev}
              disabled={currentPage === 1}
          >
              <FaChevronLeft /> {/* Single Arrow Left */}
          </button>

          {/* Page Numbers */}
          {pageRange.map((pageNum) => (
              <button
                  key={pageNum}
                  className={`btn ${pageNum === currentPage ? 'btn-primary' : ''}`}
                  onClick={() => onPageChange(pageNum)}
              >
                  {pageNum}
              </button>
          ))}

          <button
              className="btn"
              onClick={handleNext}
              disabled={currentPage === totalPages}
          >
              <FaChevronRight /> {/* Single Arrow Right */}
          </button>

          {/* Last Page Button */}
          <button
              className="btn"
              onClick={handleLast}
              disabled={currentPage === totalPages}
          >
              <FaFastForward /> {/* Double Arrow Right */}
          </button>
      </div>
  );
};

const Transaction = () => {
    const [currentPage, setCurrentPage] = useState(1);
    const [isMobile, setIsMobile] = useState(false); // State to detect mobile view
    const [transactionData, setTransactionData] = useState([]);
    const [sortConfig, setSortConfig] = useState({ key: null, direction: 'asc' }); // Sorting state
    const dataPerPage = 10; // Data per page (10 data per page)
    const [totalPages, setTotalPages] = useState(1); // Total pages from API
    const [searchTerm, setSearchTerm] = useState('');
    const [sortColumn, setSortColumn] = useState('tf.id');
    const [sortOrder, setSortOrder] = useState('asc');

    const buttonData = [
      { label: 'Copy', enabled: true },
      { label: 'CSV', enabled: true },
      { label: 'Excel', enabled: true },
      { label: 'PDF', enabled: true },
      { label: 'Print', enabled: true },
      { label: 'Column Visibility', enabled: true },
    ];

    // Fetch transaction data from API
    const fetchTransactionData = async (page, limit, search = '', sortColumn = 'tf.id', sortOrder = 'asc') => {
        try {
            console.log(`Fetching data for page: ${page}, limit: ${limit}, search: ${search}, sortColumn: ${sortColumn}, sortOrder: ${sortOrder}`); // Log page, limit, and search
            const response = await fetch(`${LOCAL_URL}/trx_face/table-log?search=${search}&sortColumn=${sortColumn}&sortOrder=${sortOrder}`, {
                method: 'GET',
                headers: { 'accept': 'application/json', 'x-api-key': API_KEY },
            });
            if (!response.ok) {
                throw new Error(`HTTP error! status: ${response.status}`);
            }
            const result = await response.json();
            const { data, total } = result.details.details.data;
            setTransactionData(data);
            setTotalPages(Math.ceil(total / dataPerPage));
        } catch (error) {
            console.error('Error fetching transaction data:', error);
        }
    };

    const fetchPaginationData = async (page, limit, search = '') => {
        try {
            console.log(`Fetching pagination data for page: ${page}, limit: ${limit}, search: ${search}`); // Log page, limit, and search
            const url = `${BASE_URL}/header_detail_param/paging?header_id=1&page=${page}&limit=${limit}&search=${search}`;
            console.log(`Fetching URL: ${url}`); // Log the URL
            const response = await fetch(url, {
                method: 'GET',
                headers: { 'accept': 'application/json', 'x-api-key': API_KEY },
            });
            if (!response.ok) {
                throw new Error(`HTTP error! status: ${response.status}`);
            }
            const result = await response.json();
            const { data } = result.details;
            setTotalPages(Math.ceil(data.length / dataPerPage));
        } catch (error) {
            console.error('Error fetching pagination data:', error);
        }
    };

    // Fetch data when component mounts and when currentPage changes
    useEffect(() => {
        fetchTransactionData(currentPage, dataPerPage, searchTerm, sortColumn, sortOrder);
        fetchPaginationData(currentPage, dataPerPage);
    }, [currentPage, searchTerm, sortColumn, sortOrder]);

    // Sorting function
    const sortData = (data, config) => {
      const { key, direction } = config;
      return [...data].sort((a, b) => {
          if (a[key] < b[key]) {
              return direction === 'asc' ? -1 : 1;
          }
          if (a[key] > b[key]) {
              return direction === 'asc' ? 1 : -1;
          }
          return 0;
      });
    };

    // Handle column header sort click
    const handleSort = (key) => {
        let direction = 'asc';
        if (sortConfig.key === key && sortConfig.direction === 'asc') {
            direction = 'desc'; // Toggle direction if the same column is clicked
        }
        setSortConfig({ key, direction });
    };

    // Get the paginated data
    const getPaginatedData = (data, page, perPage) => {
        const sortedData = sortData(data, sortConfig);
        const startIndex = (page - 1) * perPage;
        const endIndex = startIndex + perPage;
        return sortedData.slice(startIndex, endIndex);
    };

    // Handle page change
    const handlePageChange = (page) => {
        setCurrentPage(page);
    };

    // Handle search
    const handleSearch = async (event) => {
        const searchQuery = event.target.value;
        setSearchTerm(searchQuery);
        setCurrentPage(1); // Reset to first page on new search
        await fetchPaginationData(1, dataPerPage, searchQuery);
        await fetchTransactionData(1, dataPerPage, searchQuery);
    };

    // Paginated data
    const paginatedData = getPaginatedData(transactionData, currentPage, dataPerPage);

    // Detect screen size and update isMobile state
    useEffect(() => {
        const handleResize = () => {
            setIsMobile(window.innerWidth <= 768); // Change 768 to your breakpoint
        };

        handleResize();
        window.addEventListener('resize', handleResize);
        return () => window.removeEventListener('resize', handleResize);
    }, []);

    return (
        <div className="container mt-5">
            {/* Welcome Message */}
            <div className="row-card border-left border-primary shadow mb-4" style={{ backgroundColor: '#E2FBEA' }}>
                <div className="d-flex flex-column justify-content-start align-items-start p-4">
                    <div>
                        <h4 className="mb-3 text-start">
                            <i className="fas fa-warning fa-bold me-3"></i>Alert
                        </h4>
                        <p className="mb-0 text-start">
                            Get started now by creating an Application ID and explore all the demo services available on the dashboard.
                            Experience the ease and flexibility of trying out all our features firsthand.
                        </p>
                    </div>
                </div>
            </div>

            <div style={styles.contentContainer}>
                {/* Filter Form */}
                <div className="card p-3 mb-4">
                    <div className="row">
                        <div className={`col-12 ${isMobile ? 'mb-2' : 'col-md-2'}`}>
                            <label>Start Date</label>
                            <input type="date" className="form-control" />
                        </div>
                        <div className={`col-12 ${isMobile ? 'mb-2' : 'col-md-2'}`}>
                            <label>End Date</label>
                            <input type="date" className="form-control" />
                        </div>
                        <div className={`col-12 ${isMobile ? 'mb-2' : 'col-md-2'}`}>
                            <label>Application</label>
                            <select className="form-control">
                                <option>Select Application</option>
                                <option>App 1</option>
                                <option>App 2</option>
                                <option>App 3</option>
                                <option>App 4</option>
                                <option>App 5</option>
                            </select>
                        </div>
                        <div className={`col-12 ${isMobile ? 'mb-2' : 'col-md-2'}`}>
                            <label>End Point</label>
                            <select className="form-control">
                                <option>Select End Point</option>
                                <option>Endpoint 1</option>
                                <option>Endpoint 2</option>
                                <option>Endpoint 3</option>
                                <option>Endpoint 4</option>
                                <option>Endpoint 5</option>
                            </select>
                        </div>
                        <div className={`col-12 ${isMobile ? 'mb-2' : 'col-md-2'}`}>
                            <label>Service Charged</label>
                            <select className="form-control">
                                <option>Select Service</option>
                                <option>$5 - $20</option>
                                <option>$20 - $40</option>
                                <option>$40 - $60</option>
                            </select>
                        </div>
                        <div className={`col-12 ${isMobile ? 'd-flex justify-content-between' : 'col-md-2 d-flex align-items-end'}`} style={{ gap: '10px' }}>
                            <button className="btn btn-primary w-48">Apply</button>
                            <button className="btn btn-secondary w-48">Cancel</button>
                        </div>
                    </div>
                </div>

                {/* Action Buttons */}
                <div className="d-flex justify-content-between align-items-center mb-3">
                    <div>
                        {buttonData.map((button, index) =>
                            button.enabled ? (
                                <button
                                    key={index}
                                    className={`btn btn-light ${isMobile ? 'mb-2' : ''}`} // Add margin on mobile
                                    style={styles.actionButton}
                                >
                                    {button.label}
                                </button>
                            ) : null
                        )}
                    </div>

                    {/* Search Bar with Icon */}
                    <div className="input-group" style={{ width: '350px', display: 'flex', alignItems: 'center', justifyContent: 'flex-end' }}>
                        <input
                            type="text"
                            placeholder="Search Application Name or Subject ID" // Placeholder text
                            className="form-control"
                            onChange={handleSearch} // Add onChange handler
                        />
                        <span className="input-group-text">
                            <i className="fas fa-search"></i> {/* FontAwesome search icon */}
                        </span>
                    </div>
                </div>


                {/* Table */}
                <div className="table-responsive">
                    <table className="table table-bordered" style={styles.tableContainer}>
                      <thead>
                        <tr>
                          <th>
                            <button className="btn" onClick={() => handleSort('transaction_id')}>
                              Transaction ID 
                              {sortConfig.key === 'transaction_id' && 
                                  (sortConfig.direction === 'asc' ? <FaSortUp style={styles.iconMarginLeft} /> : <FaSortDown style={styles.iconMarginLeft} />)
                              }
                              {sortConfig.key !== 'transaction_id' && <FaSort style={styles.iconMarginLeft} />}
                            </button>
                          </th>
                          <th>
                            <button className="btn" onClick={() => handleSort('application_name')}>
                              Application Name 
                              {sortConfig.key === 'application_name' && 
                                  (sortConfig.direction === 'asc' ? <FaSortUp style={styles.iconMarginLeft} /> : <FaSortDown style={styles.iconMarginLeft} />)
                              }
                              {sortConfig.key !== 'application_name' && <FaSort style={styles.iconMarginLeft} />}
                            </button>
                          </th>
                          <th>
                            <button className="btn" onClick={() => handleSort('endpoint')}>
                              End Point 
                              {sortConfig.key === 'endpoint' && 
                                  (sortConfig.direction === 'asc' ? <FaSortUp style={styles.iconMarginLeft} /> : <FaSortDown style={styles.iconMarginLeft} />)
                              }
                              {sortConfig.key !== 'endpoint' && <FaSort style={styles.iconMarginLeft} />}
                            </button>
                          </th>
                          <th>
                            <button className="btn" onClick={() => handleSort('subject_id')}>
                              Subject ID 
                              {sortConfig.key === 'subject_id' && 
                                  (sortConfig.direction === 'asc' ? <FaSortUp style={styles.iconMarginLeft} /> : <FaSortDown style={styles.iconMarginLeft} />)
                              }
                              {sortConfig.key !== 'subject_id' && <FaSort style={styles.iconMarginLeft} />}
                            </button>
                          </th>
                          <th>
                            <button className="btn" onClick={() => handleSort('mode')}>
                              Mode 
                              {sortConfig.key === 'mode' && 
                                  (sortConfig.direction === 'asc' ? <FaSortUp style={styles.iconMarginLeft} /> : <FaSortDown style={styles.iconMarginLeft} />)
                              }
                              {sortConfig.key !== 'mode' && <FaSort style={styles.iconMarginLeft} />}
                            </button>
                          </th>
                          <th>
                            <button className="btn" onClick={() => handleSort('status')}>
                              Status 
                              {sortConfig.key === 'status' && 
                                  (sortConfig.direction === 'asc' ? <FaSortUp style={styles.iconMarginLeft} /> : <FaSortDown style={styles.iconMarginLeft} />)
                              }
                              {sortConfig.key !== 'status' && <FaSort style={styles.iconMarginLeft} />}
                            </button>
                          </th>
                        </tr>
                      </thead>

                      <tbody>
                        {paginatedData.length > 0 ? (
                            paginatedData.map((transaction, index) => (
                              <tr key={index}>
                                <td>{transaction.transaction_id}</td>
                                <td>{transaction.application_name}</td>
                                <td>{transaction.endpoint}</td>
                                <td>{transaction.subject_id || 'N/A'}</td>
                                <td>{transaction.mode}</td>
                                <td>{transaction.status}</td>
                              </tr>
                            ))
                        ) : (
                          <tr>
                            <td colSpan="9" className="text-center">
                              <div className="d-flex flex-column align-items-center mt-5">
                                  <img src={NoAvailable} alt="No Data Available" className="mb-3" style={styles.iconStyle} />
                                  <p>Data not available</p>
                              </div>
                            </td>
                          </tr>
                        )}
                      </tbody>

                    </table>
                </div>

                {/* Pagination */}
                <Pagination
                    currentPage={currentPage}
                    totalPages={totalPages}
                    onPageChange={handlePageChange}
                />
            </div>
        </div>
    );
};

export default Transaction;

const styles = {
  contentContainer: {
      padding: '20px',
      border: '0.1px solid rgba(0, 0, 0, 0.2)',
      borderLeft: '4px solid #0542cc',
      borderRadius: '10px',
      width: '100%',
  },
  tableContainer: {
      minHeight: '300px',
      maxHeight: '1500px',
      overflowY: 'auto',
  },
  iconStyle: {
      width: '50px',
      height: '50px',
  },
  // Add margin-left style for icons
  iconMarginLeft: {
      marginLeft: '0.7rem', // Adjust as needed
  },
};
